

#import "DbgStackFrame.h"

@implementation DbgStackFrame

/*****************************************************************************/

- (id) initWithFileName:(NSString *)ifileName 
			 moduleName:(NSString *)imoduleName
			   funcName:(NSString *)ifuncName 
			 lineNumber:(int)ilineNo
{
	
	if( (self = [super init]) != nil)
	{
		fileName = [ifileName retain];
		moduleName = [imoduleName retain];
		funcName = [ifuncName retain];
		lineNumber = [NSString stringWithFormat:@"%d", ilineNo];
		
		if([funcName isEqualToString:@"?"])
		{
			funcSignature = [NSString stringWithFormat:@"%@", 
				[fileName lastPathComponent]];
		}
		else
		{
			funcSignature = [NSString stringWithFormat:@"%@::%@", 
				[fileName lastPathComponent], funcName];
		}
		globalVariables = [[NSMutableDictionary alloc] init];
		localVariables = [[NSMutableDictionary alloc] init];
	}	
		 
	return self;
}

/*****************************************************************************/

-(void)encodeWithCoder:(NSCoder*)coder
{
    [coder encodeObject: fileName];
	[coder encodeObject: moduleName];
	[coder encodeObject: funcName];
	[coder encodeObject: lineNumber];
	[coder encodeObject: funcSignature];
	[coder encodeObject: globalVariables];
	[coder encodeObject: localVariables];
    
}

/*****************************************************************************/

-(id)initWithCoder:(NSCoder*)coder
{
	if (self=[super init]) 
	{
		fileName = [[coder decodeObject] retain];
		moduleName = [[coder decodeObject] retain];
		funcName = [[coder decodeObject] retain];
		lineNumber = [[coder decodeObject] retain];
		funcSignature = [[coder decodeObject] retain];
		globalVariables = [[coder decodeObject] retain];
		localVariables = [[coder decodeObject] retain];
	}
	return self;
}
/*****************************************************************************/

-(id) copyWithZone:(NSZone *)zone
{
	DbgStackFrame *newFrame = [[DbgStackFrame allocWithZone: zone] init];
	newFrame->fileName = [NSString stringWithString: fileName];
	newFrame->moduleName = [NSString stringWithString: moduleName];
	newFrame->funcName = [NSString stringWithString: funcName];
	newFrame->lineNumber = [NSString stringWithString: lineNumber];
	newFrame->funcSignature  = [NSString stringWithString: funcSignature];	
	
	//we can do a shallow copy here as the mutable dictionarys have imutable strings
	newFrame->globalVariables = [NSMutableArray arrayWithArray: globalVariables];
	newFrame->localVariables = [NSMutableArray arrayWithArray: localVariables];
	
	return newFrame;
}


/*****************************************************************************/

-(id) mutableCopyWithZone:(NSZone *)zone
{
	DbgStackFrame *newFrame = [[DbgStackFrame allocWithZone: zone] init];
	newFrame->fileName = [NSMutableString stringWithString: fileName];
	newFrame->moduleName = [NSMutableString stringWithString: moduleName];
	newFrame->funcName = [NSMutableString stringWithString: funcName];
	newFrame->lineNumber = [NSMutableString stringWithString: lineNumber];
	newFrame->funcSignature  = [NSMutableString stringWithString: funcSignature];	

	//we can do a shallow copy here as the mutable dictionaries have imutable strings
	newFrame->globalVariables = [NSMutableArray arrayWithArray: globalVariables];
	newFrame->localVariables = [NSMutableArray arrayWithArray: localVariables];
	
	return newFrame;
}

/*****************************************************************************/

- (void) addLocalVariable:(NSString *)name withValue:(NSString *)value
{
	[globalVariables setValue:value forKey:name];
}

/*****************************************************************************/

- (void) addGlobalVariable:(NSString *)name withValue:(NSString *)value
{
	[localVariables setValue:value forKey:name];
}

/*****************************************************************************/

- (id)replacementObjectForPortCoder:(NSPortCoder *)encoder

{
	
    if ([encoder isBycopy]) return self;
	
    return [super replacementObjectForPortCoder:encoder];
	
}

@end
